#include <stdio.h>
#include "iodefine.h"
#include "ivects.h"

#define WAIT_FOR(cond) for (i=1<<24; i; i--) if (cond) break;

int running;
static volatile int msec = 0;

static void __attribute__((interrupt))
cmt0_handler (void)
{
  msec ++;
  switch (msec % 1000)
    {
    case 500:
      PORT7.DR.BIT.B4 = 0;
      break;
    case 0:
      PORT7.DR.BIT.B4 = 1;
      break;
    }
}  

static volatile int rtc_old_usec = 0;
static volatile int rtc_period_us = 0;

static void __attribute__((interrupt))
rtc_handler (void)
{
  int h1, h2, l;
  int usec;

  h2 = msec;
  do {
    h1 = h2;
    l = CMT0.CMCNT;
    h2 = msec;
  } while (h1 != h2);

  usec = (unsigned short)h1 * 1000 + ((unsigned short)l / 48);

  rtc_period_us = usec - rtc_old_usec;
  rtc_old_usec = usec;
}

static start_clock ()
{
  int i;
  /* Real Time Clock */
  SYSTEM.SUBOSCCR.BIT.SUBSTOP = 0;
  running = 1;

  PORT7.DR.BYTE = ~0;

  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);
  RTC.RCR2.BIT.START = 0;
  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);
  WAIT_FOR (! RTC.RCR2.BIT.START);

  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);
  RTC.RCR2.BIT.RESET = 1;
  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);
  WAIT_FOR (!RTC.RCR2.BIT.RESET)
  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);

  RTC.RSECCNT.BYTE = 0;
  RTC.RMINCNT.BYTE = 0;
  RTC.RHRCNT.BYTE = 0;
  RTC.RDAYCNT.BYTE = 0;
  RTC.RMONCNT.BYTE = 9;
  RTC.RYRCNT.WORD = 0x2010;
  RTC.RSECCNT.BYTE = 0;

  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);
  RTC.RCR2.BIT.START = 1;
  printf("%d: RCR2 = 0x%02x\n", __LINE__, RTC.RCR2.BYTE);
  WAIT_FOR (RTC.RCR2.BIT.START);

  _vectors[VECT(RTC,PRD)] = rtc_handler;
  IEN(RTC,PRD) = 1;
  IPR(RTC,PRD) = 5;
  RTC.RCR1.BIT.PES = 6;
  RTC.RCR1.BIT.PIE = 1;
}

main()
{
  int i, j;
  int old_64 = -1;
  int old_sec = -1;

  /* We start with a 12 MHz crystal.  */
  /* 0 = x8  1 = x4  2 = x2  3 = x1 */
  /* ICLK: x8 =  96 MHz  */
  /* PCLK: x4 =  48 MHz   */
  /* BCLK: x4 =  48 MHz   */
  /*                     I B P */
  SYSTEM.SCKCR.LONG = 0x00010100;
  /*	0 = 96 MHz
	1 = 48 MHz
	2 = 24 MHz
	3 = 12 MHz
  */

  /* Make sure 32768 Hz clock is running.  */
  SYSTEM.SUBOSCCR.BIT.SUBSTOP = 1;

  PORT7.DR.BYTE = 0;
  PORT7.DDR.BYTE = ~0;

  MSTP(CMT0) = 0;
  _vectors[VECT(CMT0,CMI0)] = cmt0_handler;
  IEN(CMT0,CMI0) = 1;
  IPR(CMT0,CMI0) = 5;

  CMT0.CMCR.BIT.CKS = 0;  /* 48 MHz */
  CMT0.CMCR.BIT.CMIE = 1; /* interrupts */
  CMT0.CMCOR = 48000 / 8;
  CMT.CMSTR0.BIT.STR0 = 1; /* start */

  tty_init ();
  tty_puts("\nHello, world\n");

  start_clock ();

  i = 0;
  while (1)
    {
      int new_64 = RTC.R64CNT.BYTE;
      if (new_64 != old_64) {
	old_64 = new_64;
	printf("  %02x:%02x.%02x  %7d usec/sec \r", RTC.RMINCNT.BYTE, RTC.RSECCNT.BYTE, new_64, rtc_period_us);
	fflush (stdout);
	PORT7.DR.BIT.B5 = (new_64 & 0x40) ? 1 : 0;
      }

      if (tty_ready())
	{
	  switch (tty_getc())
	    {
	    case ' ':
	      if (running)
		{
		  SYSTEM.SUBOSCCR.BIT.SUBSTOP = 1;
		  running = 0;
		  tty_puts("Stopped\n");
		}
	      else
		{
		  tty_puts("Running\n");
		  start_clock ();
		}
	      break;
	      
	    }
	}
    }
}
