/* This is actually Renesas's ID, but since it's their chips... */
#define MY_VENDOR_ID 0x45b

typedef struct {
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t bcdUSB;
  uint8_t bDeviceClass;
  uint8_t bDeviceSubClass;
  uint8_t bDeviceProtocol;
  uint8_t bMaxPacketSize0;
  uint16_t idVendor;
  uint16_t idProduct;
  uint16_t bcdDevice;
  uint8_t iManufacturer;
  uint8_t iProduct;
  uint8_t iSerialNumber;
  uint8_t bNumConfigurations;
} USB_device_descriptor;

typedef struct {
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t wTotalLength;
  uint8_t bNumInterfaces;
  uint8_t bConfigurationValue;
  uint8_t iConfiguration;
  uint8_t bmAttributes;
  uint8_t MaxPower;
} USB_configuration_descriptor;

typedef struct {
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint8_t bInterfaceNumber;
  uint8_t bAlternateSetting;
  uint8_t bNumEndpoints;
  uint8_t bInterfaceClass;
  uint8_t bInterfaceSubClass;
  uint8_t bInterfaceProtocol;
  uint8_t iInterface;
} USB_interface_descriptor;

typedef struct {
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint8_t bEndpointAddress;
  uint8_t bmAttributes;
  uint16_t wMaxPacketSize;
  uint8_t bInterval;
} USB_endpoint_descriptor;

typedef struct {
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t wLANGID[];
} USB_lang_descriptor;

typedef struct {
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t bString[];
} USB_string_descriptor;

#define EP_OUT		0x00
#define EP_IN		0x80
#define EP_CONTROL	0
#define EP_ISOCHRONOUS	1
#define EP_BULK		2
#define EP_INTERRUPT	3

#define CONFIG_ATTR_RESERVED_SET 0x80
#define CONFIG_ATTR_SELF_POWERED 0x40
#define CONFIG_ATTR_REMOTE_WAKE  0x20

#define RQ_GET_STATUS	0
#define RQ_CLEAR_FEATURE	1
#define RQ_SET_FEATURE	3
#define RQ_SET_ADDRESS	5
#define	RQ_GET_DESCRIPTOR	6
#define	RQ_SET_DESCRIPTOR	7
#define	RQ_GET_CONFIGURATION	8
#define	RQ_SET_CONFIGURATION	9
#define	RQ_GET_INTERFACE	10
#define	RQ_SET_INTERFACE	11
#define	RQ_SYNC_FRAME	12

#define DESCRIPTOR_DEVICE		1
#define DESCRIPTOR_CONFIGURATION	2
#define DESCRIPTOR_STRING		3
#define DESCRIPTOR_INTERFACE		4
#define DESCRIPTOR_ENDPOINT		5
#define DESCRIPTOR_DEV_QUALIFIER	6

extern volatile int usb_connected;

void usb_init (void);
void usb_uninit (void);

void usb_write_start (int pipe);
void usb_write_data (const void *vdata, int len);
void usb_write_done (void);
void usb_write_more (void);
void usb_setup_ack (void);

/* Shortcut for start/data/done */
void usb_write_pipe (int pipe, const void *data, int len);

/* Returns number of bytes actually read, will block if no data is
   available. */
int usb_read (unsigned char *buf, int maxlen);

/* Doesn't return until all bytes are written or queued.  */
int usb_write (unsigned char *buf, int len);
